package com.blackshark.gamecenter.sdk.demo.shark;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.ActivityInfo;
import android.graphics.Color;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.os.Process;
import android.text.TextUtils;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.blackshark.gamecenter.base.constant.AppConstant;
import com.blackshark.gamecenter.sdk.OnAuthInfoListener;
import com.blackshark.gamecenter.sdk.OnLoginProcessListener;
import com.blackshark.gamecenter.sdk.SharkErrorCode;
import com.blackshark.gamecenter.sdk.SharkOpenPlatform;
import com.blackshark.gamecenter.sdk.entry.GameRoleInfo;
import com.blackshark.gamecenter.sdk.entry.ScreenOrientation;
import com.blackshark.gamecenter.sdk.entry.SharkAccountInfo;

public class MainActivity extends Activity implements OnLoginProcessListener {

    public static final int MSG_REPEATPAY = 10000;
    public static final int MSG_UNREPEATPAY = 20000;
    public static final int MSG_LOGIN_SUCCESS = 30000;
    public static final int MSG_LOGIN_FAILED = 40000;
    public static final int MSG_DO_NOT_REPEAT_OPERATION = 70000;

    public static Handler mUIHandler = new Handler(Looper.getMainLooper());
    /**
     * Screen orientation switch
     */
    private Button screenButton;

    /**
     * Display screen orientation
     */
    private TextView screenTextView;

    /**
     * Screen orientation property
     */
    private ScreenOrientation orientation = ScreenOrientation.vertical;

    private int demoScreenOrientation = ActivityInfo.SCREEN_ORIENTATION_PORTRAIT;
    private static SharkAccountInfo accountInfo;
    private String session;
    private String msg;

    private Handler handler = new Handler(new HandlerCallback());
    private Button btnLogin;

    private class HandlerCallback implements Handler.Callback {

        @Override
        public boolean handleMessage(Message msg) {
            switch (msg.what) {
                case MSG_LOGIN_SUCCESS:
                    Log.e("SharkGameSDK", getResources().getString(R.string.login_success)
                            + "\nuid:" + accountInfo.getUid()
                            + "\nsessionId:" + accountInfo.getSession()
                            + "\nnikeName:" + accountInfo.getNickName());
                    Toast.makeText(MainActivity.this, getResources().getString(R.string.login_success)
                            + "\nuid:" + accountInfo.getUid()
                            + "\nsessionId:" + accountInfo.getSession()
                            + "\nnikeName:" + accountInfo.getNickName(), Toast.LENGTH_SHORT).show();
                    break;
                case MSG_LOGIN_FAILED:
                    Toast.makeText(MainActivity.this, getResources().getString(R.string.login_failed), Toast.LENGTH_SHORT).show();
                    break;
                case MSG_DO_NOT_REPEAT_OPERATION:
                    Toast.makeText(MainActivity.this, getResources().getString(R.string.demo_islogin), Toast.LENGTH_SHORT).show();
                    break;
                default:
                    break;
            }
            return false;
        }
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        if (Build.VERSION.SDK_INT >= 28) {
            getWindow().getDecorView().setSystemUiVisibility(
                    View.SYSTEM_UI_FLAG_FULLSCREEN
                            | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN);
            WindowManager.LayoutParams lp =
                    getWindow().getAttributes();
            lp.layoutInDisplayCutoutMode =
                    WindowManager.LayoutParams
                            .LAYOUT_IN_DISPLAY_CUTOUT_MODE_SHORT_EDGES;
            getWindow().setAttributes(lp);
        }



        btnLogin = (Button) findViewById(R.id.btn_login);
        btnLogin.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                SharkOpenPlatform.getInstance().login(MainActivity.this, MainActivity.this);
            }
        });

        Button btnPay = (Button) findViewById(R.id.btn_money_payment);
        btnPay.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (accountInfo == null) {
                    Toast.makeText(MainActivity.this, "请先登录", Toast.LENGTH_SHORT).show();
                    return;
                }
                Intent intent = new Intent(MainActivity.this, OrderActivity.class);
                Bundle bundle = new Bundle();
                bundle.putString("_uid", accountInfo.getUid());
                intent.putExtra("_bundle", bundle);
                startActivity(intent);
            }
        });

        Button btnLogout = (Button) findViewById(R.id.btn_exit);
        btnLogout.setOnClickListener(v -> {
            exit();
        });

        Button btnSubmitRole = (Button) findViewById(R.id.btn_submit_role_data);
        btnSubmitRole.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (accountInfo == null) {
                    Toast.makeText(MainActivity.this, "请先登录", Toast.LENGTH_SHORT).show();
                    return;
                }

                GameRoleInfo info = new GameRoleInfo();
                info.setRoleId("404");
                info.setRoleName("一鸣");
                info.setRoleLevel(99);
                info.setRolePower(999);
                info.setRoleServerId("1000");
                info.setRoleServerName("国服001");
                info.setRoleZoneId("b1");
                info.setRoleZoneName("桃花岛");
                SharkOpenPlatform.getInstance().submitRoleInfo(MainActivity.this, info);
            }
        });

        Button btnAuth = (Button) findViewById(R.id.btn_authentication);
        btnAuth.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                SharkOpenPlatform.getInstance().getAuthenticationInfo(MainActivity.this, new OnAuthInfoListener() {
                    @Override
                    public void onSuccess(boolean isCertified, int age) {
                        runOnUiThread(() -> Toast.makeText(MainActivity.this, isCertified ? "已实名认证，年龄：" + age : "用户未实名认证", Toast.LENGTH_SHORT).show());
                    }

                    @Override
                    public void onError(int i, String s) {
                        runOnUiThread(() -> Toast.makeText(MainActivity.this, "获取实名认证信息失败! code: " + i + " msg: " + s, Toast.LENGTH_SHORT).show());
                    }
                });
            }
        });

        screenTextView = (TextView) findViewById(R.id.text_screen);
        screenTextView.setTextColor(Color.BLACK);
        screenTextView.setTextSize(18);
        if (orientation == ScreenOrientation.horizontal)
            screenTextView.setText(getResources().getString(R.string.screen_h));
        else
            screenTextView.setText(getResources().getString(R.string.screen_v));

        screenButton = (Button) findViewById(R.id.btn_change_screen);
        screenButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (orientation == ScreenOrientation.horizontal) {
                    screenTextView.setText(getResources().getString(R.string.screen_v));
                    demoScreenOrientation = ActivityInfo.SCREEN_ORIENTATION_PORTRAIT;
                    orientation = ScreenOrientation.vertical;
                } else {
                    screenTextView.setText(getResources().getString(R.string.screen_h));
                    demoScreenOrientation = ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE;
                    orientation = ScreenOrientation.horizontal;
                }
                MainActivity.this.setRequestedOrientation(demoScreenOrientation);
            }
        });

        setGameAppConfig();

        runOnUiThread(this::alertUserAgreement);
    }

    /**
     * 重写 Activity 的 onKeyDown 方法，拦截返回键事件，已登录则必须调用SDK的退出接口，由SDK处理退出逻辑；未登录则由游戏自己处理退出逻辑
     */
    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            if (!TextUtils.isEmpty(session)) {
                exit();
                return true;
            }
        }
        return super.onKeyDown(keyCode, event);
    }

    //修改 appId 与 appKey  并缓存
    private void setGameAppConfig() {

        EditText appIdEt = findViewById(R.id.et_change_appId);
        EditText appKeyEt = findViewById(R.id.et_change_appKey);
        Button applyBtn = findViewById(R.id.btn_message_apply);
        Button resetBtn = findViewById(R.id.btn_message_reset);

        SharedPreferences sp = getSharedPreferences("userAppConfig", Context.MODE_PRIVATE);
        appIdEt.setText(DemoApp.gameInfo.getAppId());
        appKeyEt.setText(DemoApp.gameInfo.getAppKey());

        applyBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                String appIdStr = appIdEt.getText().toString();
                String appKeyStr = appKeyEt.getText().toString();

                if (TextUtils.isEmpty(appIdStr)) {
                    Toast.makeText(MainActivity.this, "请输入appId", Toast.LENGTH_SHORT).show();
                    return;
                }
                if (TextUtils.isEmpty(appKeyStr)) {
                    Toast.makeText(MainActivity.this, "请输入appKey", Toast.LENGTH_SHORT).show();
                    return;
                }

                SharedPreferences.Editor edit = sp.edit();

                edit.putString("appId", appIdStr);
                edit.putString("appKey", appKeyStr);

                edit.apply();

                Toast.makeText(MainActivity.this, "即将重启", Toast.LENGTH_SHORT).show();
                handler.postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        restartActivity();
                    }
                }, 1000);
            }
        });

        resetBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                SharedPreferences.Editor edit = sp.edit();
                edit.remove("appId");
                edit.remove("appKey");
                edit.apply();

                Toast.makeText(MainActivity.this, "即将重启", Toast.LENGTH_SHORT).show();
                handler.postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        restartActivity();
                    }
                }, 1000);
            }
        });
    }

    /**
     * 重新启动app
     */
    private void restartActivity() {
        Intent intent = new Intent(this, SplashActivity.class);
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        startActivity(intent);
        Process.killProcess(Process.myPid());
    }

    /**
     * ⽤⼾在第⼀次打开游戏时，需要显⽰相关隐私协议，⽤⼾选择之后，调⽤SharkOpenPlatform.getInstance().onUserAgreed(Activity activity)；告知SDK⽤⼾是否同意隐私协议。
     * <p>
     * 该接⼝必须在⽤⼾选择同意隐私协议(游戏⾃⼰的隐私协议)之后调⽤，否则⽆法正常登录；若⽤⼾拒绝隐私协议，不可调⽤该接⼝，且⽆法登录。
     * <p>
     * 注：SDK本⾝不会弹出任何隐私协议，隐私协议游戏内⾃⾏设计实现。
     */
    private void alertUserAgreement() {
        SharedPreferences sp = getSharedPreferences("userAgreementResult", 0);
        boolean agreed = sp.getBoolean("userAgreementResult", false);
        if (agreed) {
            mockCpLogin();
            return;
        }
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle("重要提示");
        String message = "请同意用户协议";
        builder.setMessage(message);
        builder.setPositiveButton("同意", (dialog, which) -> {
            mockCpLogin();

            SharedPreferences.Editor editor = sp.edit();
            editor.putBoolean("userAgreementResult", true);
            editor.apply();
            dialog.dismiss();
        });
        builder.setNegativeButton("拒绝", (dialog, which) -> {
            //用户不同意隐私协议，不通过，Demo在此处退出游戏，接入方可按自己要求处理，但不能调用SharkOpenPlatform.getInstance().onUserAgreed(this);
            dialog.dismiss();
            Process.killProcess(Process.myPid());
        });
        builder.create().show();
    }

    private void mockCpLogin() {
        //用户已经同意过隐私协议，直接通过
        SharkOpenPlatform.getInstance().onUserAgreed(this);

        //模拟游戏CP 自动登录逻辑
        SharkOpenPlatform.getInstance().login(MainActivity.this, MainActivity.this);
    }

    @Override
    public void finishLoginProcess(int errCode, SharkAccountInfo sharkAccountInfo) {
        if (SharkErrorCode.SHARK_LOGIN_SUCCESS == errCode) {
            accountInfo = sharkAccountInfo;
            session = sharkAccountInfo.getSession();
            handler.sendEmptyMessage(MSG_LOGIN_SUCCESS);
        } else if (SharkErrorCode.SHARK_LOGIN_ERROR_REPEAT_OPERATION == errCode) {
            handler.sendEmptyMessage(MSG_DO_NOT_REPEAT_OPERATION);
        } else {
            handler.sendEmptyMessage(MSG_LOGIN_FAILED);
        }
    }

    /**
     * SDK自带了退出弹窗，游戏不需要自己实现，推荐重写activity的onKeyDown方法，当用户未登录时，游戏自己处理退出逻辑；
     * 用户已登录则调用SDK接口处理退出逻辑。帐号退出或注销时，在适当的地方调用退出接口。
     */
    private void exit() {
        if (!TextUtils.isEmpty(session)) {
            SharkOpenPlatform.getInstance().appExit(MainActivity.this, code -> {
                Log.e("errorCode===", code + "");
                if (code == AppConstant.PROMOTE_ON_WINDOW_EXIT) {
                    Process.killProcess(Process.myPid());
                }
            });
        } else {
            finish();
        }
    }
}
