package com.blackshark.gamecenter.sdk.demo.shark;

import android.app.Activity;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import com.blackshark.gamecenter.sdk.OnPayProcessListener;
import com.blackshark.gamecenter.sdk.SharkErrorCode;
import com.blackshark.gamecenter.sdk.SharkOpenPlatform;
import com.blackshark.gamecenter.sdk.entry.SharkBuyInfo;

import java.util.Map;

public class OrderActivity extends Activity {
    private static final String TAG = OrderActivity.class.getSimpleName();

    private EditText etProductId, etProductName, etProductDesc,
            etPrice, edBuyNum, etOrderId, etNotifyUrl;

    private String productId, productName, productDesc, orderId, notifyUrl, uid;
    private int price, buyNum;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_order);
        etProductId = findViewById(R.id.et_product_id);
        etProductName = findViewById(R.id.et_product_name);
        etProductDesc = findViewById(R.id.et_product_desc);
        etPrice = findViewById(R.id.et_price);
        edBuyNum = findViewById(R.id.et_buynum);
        etOrderId = findViewById(R.id.et_order_id);
        etNotifyUrl = findViewById(R.id.et_notify_url);
        etNotifyUrl.setText("https://test-unionpay.blackshark.com/v1/pay/test_report_cp");

        Bundle bundle = getIntent().getBundleExtra("_bundle");
        if (bundle == null) {
            Log.e(TAG, " bundle is null, return");
            return;
        }
        uid = bundle.getString("_uid");

        Button btnAgree = findViewById(R.id.btn_agreed);
        btnAgree.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                SharkOpenPlatform.getInstance().onUserAgreed(OrderActivity.this);
            }
        });
    }

    private void getOrderInfo() {
        productId = etProductId.getText().toString().trim();
        productName = etProductName.getText().toString().trim();
        productDesc = etProductDesc.getText().toString().trim();
        orderId = etOrderId.getText().toString().trim();

        price = (int) (Float.parseFloat(etPrice.getText().toString().trim()) * 100);
        buyNum = Integer.parseInt(edBuyNum.getText().toString().trim());
        notifyUrl = etNotifyUrl.getText().toString().trim();
    }

    private boolean checkParams() {
        if (TextUtils.isEmpty(productId)) {
            ToastUtils.showShort("商品ID为空");
            return false;
        }
        if (TextUtils.isEmpty(productName)) {
            ToastUtils.showShort("商品名称为空");
            return false;
        }
        if (TextUtils.isEmpty(orderId)) {
            ToastUtils.showShort("订单号为空");
            return false;
        }
        if (price < 0f) {
            ToastUtils.showShort("商品价格不可小于0");
            return false;
        }
        if (buyNum <= 0) {
            ToastUtils.showShort("购买数量不可小于0");
            return false;
        }
        if (TextUtils.isEmpty(notifyUrl)) {
            ToastUtils.showShort("商户通知地址为空");
            return false;
        }
        return true;
    }

    /**
     * 哪些参数必填，哪些参数可为空，哪些参数不参与签名，文档中都有详细说明
     * <p>
     * 如对签名机制还有疑问，可直接拷贝demo中的签名方法到项目中使用
     */
    private SharkBuyInfo getOrder() {
        SharkBuyInfo payParams = new SharkBuyInfo();
        payParams.setAppId(DemoApp.gameInfo.getAppId()); // 游戏 ID(不能为空)，必填
        payParams.setProductId(productId); // CP 游戏道具 ID，可为空，必填
        payParams.setProductName(productName); // 游戏道具名称(不能为空)，必填
        payParams.setProductDesc(productDesc); // 游戏道具说明，可为空，必填
        payParams.setOrderId(orderId); // CP 订单 ID(不能为空)，必填
        payParams.setPrice(price); // 总金额，单位 分(不能为空)，必填
        payParams.setBuyNum(buyNum); // 道具购买的数量，不参与签名，必填
        payParams.setPayNotifyUrl(notifyUrl); // 支付回调Url(不能为空)，必填

        long current = System.currentTimeMillis();
        payParams.setTimeStart(DateUtils.formatYMDHMS(current)); // 订单创建时间，格式为 yyyyMMddHHmmss，可为空，必填
        payParams.setTimeExpire(DateUtils.formatYMDHMS(current + 30 * 1000 * 60L)); // 订单超时时间，格式为 yyyyMMddHHmmss，可为空，必填
        payParams.setUid(uid); // 黑鲨游戏账户用户 ID(不能为空)，必填

        Map<String, String> orderParam = OrderUtils.buildOrderParam(payParams);
        String sign = OrderUtils.getSign(orderParam, DemoApp.app.getGameAppSecret()); // 这里使用申请的AppSecret进行签名
        payParams.setSign(sign); // 参数签名(不能为空)，必填

        Log.i(TAG, "payParams: " + payParams);
        return payParams;
    }

    public void pay(View view) {
        getOrderInfo();
        if (!checkParams()) {
            Log.e(TAG, "params check failed");
            return;
        }
        SharkOpenPlatform.getInstance().uniPay(this, getOrder(), new OnPayProcessListener() {
            @Override
            public void finishPayProcess(int code) {
                switch (code) {
                    case SharkErrorCode.SHARK_PAYMENT_SUCCESS:
                        showToast(getResources().getString(R.string.demo_payment_success));
                        break;
                    case SharkErrorCode.SHARK_PAYMENT_ERROR_PAY_CANCEL:
                        showToast(getResources().getString(R.string.demo_payment_cancel));
                        break;
                    case SharkErrorCode.SHARK_PAYMENT_ERROR_LOGIN_FAIL:
                        showToast(getResources().getString(R.string.demo_payment_please_login));
                        break;
                    case SharkErrorCode.SHARK_PAYMENT_ERROR_ACTION_EXECUTED:
                        showToast(getResources().getString(R.string.demo_islogin));
                        break;
                    case SharkErrorCode.SHARK_PAYMENT_ERROR_PAY_FAILURE:
                    default:
                        showToast(getResources().getString(R.string.demo_payment_failed));
                        break;
                }
            }
        });
    }

    public void createOrder(View view) {
        String orderID = "demoOrder" + System.currentTimeMillis();
        etOrderId.setText(orderID);
    }

    private void showToast(String msg) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(OrderActivity.this, msg, Toast.LENGTH_LONG).show();
            }
        });
    }
}