package com.blackshark.gamecenter.sdk.demo.shark;

import android.util.Log;

import com.blackshark.gamecenter.sdk.entry.SharkBuyInfo;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class OrderUtils {

    private static final String TAG = "OrderUtils";

    /**
     * 构造支付订单参数列表
     */
    public static Map<String, String> buildOrderParam(SharkBuyInfo payParams) {
        Map<String, String> keyValues = new HashMap<String, String>();

        keyValues.put("app_id", payParams.getAppId());

        keyValues.put("out_trade_no", payParams.getOrderId());

        keyValues.put("time_start", payParams.getTimeStart());

        keyValues.put("time_expire", payParams.getTimeExpire());

        keyValues.put("total_amount", payParams.getPrice() + "");

        keyValues.put("product_code", payParams.getProductId());

        keyValues.put("product_body", payParams.getProductName());

        keyValues.put("product_detail", payParams.getProductDesc());

        keyValues.put("uid", payParams.getUid());

        keyValues.put("notify_url", payParams.getPayNotifyUrl());

        return keyValues;
    }

    /**
     * 拼接键值对
     */
    private static String buildKeyValue(String key, String value, boolean isEncode) {
        StringBuilder sb = new StringBuilder();
        sb.append(key);
        sb.append("=");
        if (isEncode) {
            try {
                sb.append(URLEncoder.encode(value, "UTF-8"));
            } catch (UnsupportedEncodingException e) {
                sb.append(value);
            }
        } else {
            sb.append(value);
        }
        return sb.toString();
    }

    /**
     * 对支付参数信息进行签名
     *
     * @param map 待签名授权信息
     */
    public static String getSign(Map<String, String> map, String appKey) {
        List<String> keys = new ArrayList<String>(map.keySet());
        // key排序
        Collections.sort(keys);

        StringBuilder authInfo = new StringBuilder();
        for (int i = 0; i < keys.size() - 1; i++) {
            String key = keys.get(i);
            String value = map.get(key);
            authInfo.append(buildKeyValue(key, value, false));
            authInfo.append("&");
        }

        String tailKey = keys.get(keys.size() - 1);
        String tailValue = map.get(tailKey);
        authInfo.append(buildKeyValue(tailKey, tailValue, false));

        authInfo.append("&app_key=").append(appKey); // 关键，将appKey添加到排序后的订单
        String orderStr = authInfo.toString();
        Log.i(TAG, "orderStr: " + orderStr);
        try {
            return SignUtil.getMD5(orderStr).toUpperCase();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "";
    }
}
